@@ -3,12 +3,18 @@ import * as FuzzySearch from 'fuzzy-search'
 import fetch from 'node-fetch'
 import * as URI from 'urijs'
 import * as url from 'url'
-import { promisify } from 'util'
+import { isDeepStrictEqual, promisify } from 'util'
 import * as LSP from 'vscode-languageserver/node'
 import { TextDocument } from 'vscode-languageserver-textdocument'
 import * as Parser from 'web-tree-sitter'
 
-import { flattenArray, flattenObjectValues } from './util/flatten'
+import {
+  getAllDeclarationsInTree,
+  getGlobalDeclarations,
+  getLocalDeclarations,
+  GlobalDeclarations,
+} from './util/declarations'
+import { flattenArray } from './util/flatten'
 import { getFilePaths } from './util/fs'
 import { analyzeShebang } from './util/shebang'
 import * as sourcing from './util/sourcing'
@@ -16,36 +22,22 @@ import * as TreeSitterUtil from './util/tree-sitter'
 
 const readFileAsync = promisify(fs.readFile)
 
-type Kinds = { [type: string]: LSP.SymbolKind }
-
-type Declarations = { [word: string]: LSP.SymbolInformation[] }
-type FileDeclarations = { [uri: string]: Declarations }
-
-type Trees = { [uri: string]: Parser.Tree }
-type Texts = { [uri: string]: string }
+type AnalyzedDocument = {
+  document: TextDocument
+  globalDeclarations: GlobalDeclarations
+  sourcedUris: Set<string>
+  tree: Parser.Tree
+}
 
 /**
  * The Analyzer uses the Abstract Syntax Trees (ASTs) that are provided by
  * tree-sitter to find definitions, reference, etc.
  */
 export default class Analyzer {
-  private parser: Parser
   private console: LSP.RemoteConsole
-  private uriToTextDocument: { [uri: string]: TextDocument } = {}
-  private uriToTreeSitterTrees: Trees = {}
-
-  // We need this to find the word at a given point etc.
-  private uriToFileContent: Texts = {}
-  private uriToDeclarations: FileDeclarations = {}
-  private uriToSourcedUris: { [uri: string]: Set<string> } = {}
-  private treeSitterTypeToLSPKind: Kinds = {
-    // These keys are using underscores as that's the naming convention in tree-sitter.
-    environment_variable_assignment: LSP.SymbolKind.Variable,
-    function_definition: LSP.SymbolKind.Function,
-    variable_assignment: LSP.SymbolKind.Variable,
-  }
-
   private includeAllWorkspaceSymbols: boolean
+  private parser: Parser
+  private uriToAnalyzedDocument: Record<string, AnalyzedDocument | undefined> = {}
   private workspaceFolder: string | null
 
   public constructor({
@@ -65,12 +57,63 @@ export default class Analyzer {
     this.workspaceFolder = workspaceFolder
   }
 
+  /**
+   * Analyze the given document, cache the tree-sitter AST, and iterate over the
+   * tree to find declarations.
+   *
+   * Returns all, if any, syntax errors that occurred while parsing the file.
+   *
+   */
+  public analyze({
+    document,
+    uri, // NOTE: we don't use document.uri to make testing easier
+  }: {
+    document: TextDocument
+    uri: string
+  }): LSP.Diagnostic[] {
+    const fileContent = document.getText()
+
+    const tree = this.parser.parse(fileContent)
+
+    const { diagnostics, globalDeclarations } = getGlobalDeclarations({ tree, uri })
+
+    this.uriToAnalyzedDocument[uri] = {
+      document,
+      globalDeclarations,
+      sourcedUris: sourcing.getSourcedUris({
+        fileContent,
+        fileUri: uri,
+        rootPath: this.workspaceFolder,
+        tree,
+      }),
+      tree,
+    }
+
+    function findMissingNodes(node: Parser.SyntaxNode) {
+      if (node.isMissing()) {
+        diagnostics.push(
+          LSP.Diagnostic.create(
+            TreeSitterUtil.range(node),
+            `Syntax error: expected "${node.type}" somewhere in the file`,
+            LSP.DiagnosticSeverity.Warning,
+          ),
+        )
+      } else if (node.hasError()) {
+        node.children.forEach(findMissingNodes)
+      }
+    }
+
+    findMissingNodes(tree.rootNode)
+
+    return diagnostics
+  }
+
   /**
    * Initiates a background analysis of the files in the workspaceFolder to
    * enable features across files.
    *
-   * NOTE: if the source aware feature works well, we can likely remove this and
-   * rely on parsing files as they are sourced.
+   * NOTE that when the source aware feature is enabled files are also parsed
+   * when they are found.
    */
   public async initiateBackgroundAnalysis({
     backgroundAnalysisMaxFiles,
@@ -151,20 +194,21 @@ export default class Analyzer {
   }
 
   /**
-   * Find all the locations where something has been defined.
+   * Find all the locations where the word was declared.
    */
-  public findDefinition({
+  public findDeclarationLocations({
     position,
     uri,
     word,
   }: {
-    position?: { line: number; character: number }
+    position: LSP.Position
     uri: string
     word: string
   }): LSP.Location[] {
-    const tree = this.uriToTreeSitterTrees[uri]
+    const tree = this.uriToAnalyzedDocument[uri]?.tree
+
     if (position && tree) {
-      // NOTE: when a word is a file path to a sourced file, we return a location it.
+      // NOTE: when a word is a file path to a sourced file, we return a location to it.
       const sourcedLocation = sourcing.getSourcedLocation({
         position,
         rootPath: this.workspaceFolder,
@@ -177,27 +221,128 @@ export default class Analyzer {
       }
     }
 
-    const fileDeclarations = this.getReachableUriToDeclarations({ uri })
-
-    return Object.keys(fileDeclarations)
-      .reduce((symbols, uri) => {
-        const declarationNames = fileDeclarations[uri][word] || []
-        declarationNames.forEach((d) => symbols.push(d))
-        return symbols
-      }, [] as LSP.SymbolInformation[])
-      .map((symbol) => symbol.location)
+    return this.findDeclarationsMatchingWord({
+      exactMatch: true,
+      position,
+      uri,
+      word,
+    }).map((symbol) => symbol.location)
   }
 
   /**
-   * Find all the symbols matching the query using fuzzy search.
+   * Find all the declaration symbols in the workspace matching the query using fuzzy search.
    */
-  public search(query: string): LSP.SymbolInformation[] {
-    const searcher = new FuzzySearch(this.getAllSymbols(), ['name'], {
+  public findDeclarationsWithFuzzySearch(query: string): LSP.SymbolInformation[] {
+    const searcher = new FuzzySearch(this.getAllDeclarations(), ['name'], {
       caseSensitive: true,
     })
     return searcher.search(query)
   }
 
+  /**
+   * Find declarations for the given word and position.
+   */
+  public findDeclarationsMatchingWord({
+    exactMatch,
+    position,
+    uri,
+    word,
+  }: {
+    exactMatch: boolean
+    position: LSP.Position
+    uri: string
+    word: string
+  }): LSP.SymbolInformation[] {
+    return this.getAllDeclarations({ uri, position }).filter((symbol) => {
+      if (exactMatch) {
+        return symbol.name === word
+      } else {
+        return symbol.name.startsWith(word)
+      }
+    })
+  }
+
+  /**
+   * Find all the locations where the given word was defined or referenced.
+   *
+   * FIXME: take position into account
+   * FIXME: take file into account
+   */
+  public findReferences(word: string): LSP.Location[] {
+    const uris = Object.keys(this.uriToAnalyzedDocument)
+    return flattenArray(uris.map((uri) => this.findOccurrences(uri, word)))
+  }
+
+  /**
+   * Find all occurrences (references or definitions) of a word in the given file.
+   * It's currently not scope-aware.
+   *
+   * FIXME: should this take the scope into account? I guess it should
+   * as this is used for highlighting.
+   */
+  public findOccurrences(uri: string, word: string): LSP.Location[] {
+    const analyzedDocument = this.uriToAnalyzedDocument[uri]
+    if (!analyzedDocument) {
+      return []
+    }
+
+    const { tree } = analyzedDocument
+
+    const locations: LSP.Location[] = []
+
+    TreeSitterUtil.forEach(tree.rootNode, (n) => {
+      let namedNode: Parser.SyntaxNode | null = null
+
+      if (TreeSitterUtil.isReference(n)) {
+        namedNode = n.firstNamedChild || n
+      } else if (TreeSitterUtil.isDefinition(n)) {
+        namedNode = n.firstNamedChild
+      }
+
+      if (namedNode && namedNode.text === word) {
+        const range = TreeSitterUtil.range(namedNode)
+
+        const alreadyInLocations = locations.some((loc) => {
+          return isDeepStrictEqual(loc.range, range)
+        })
+
+        if (!alreadyInLocations) {
+          locations.push(LSP.Location.create(uri, range))
+        }
+      }
+    })
+
+    return locations
+  }
+
+  public getAllVariables({
+    position,
+    uri,
+  }: {
+    position: LSP.Position
+    uri: string
+  }): LSP.SymbolInformation[] {
+    return this.getAllDeclarations({ uri, position }).filter(
+      (symbol) => symbol.kind === LSP.SymbolKind.Variable,
+    )
+  }
+
+  /**
+   * Get all symbol declarations in the given file. This is used for generating an outline.
+   *
+   * TODO: convert to DocumentSymbol[] which is a hierarchy of symbols found in a given text document.
+   */
+  public getDeclarationsForUri({ uri }: { uri: string }): LSP.SymbolInformation[] {
+    const tree = this.uriToAnalyzedDocument[uri]?.tree
+
+    if (!tree?.rootNode) {
+      return []
+    }
+
+    return getAllDeclarationsInTree({ uri, tree })
+  }
+
+  // TODO: move somewhere else than the analyzer...
   public async getExplainshellDocumentation({
     params,
     endpoint,
@@ -205,13 +350,17 @@ export default class Analyzer {
     params: LSP.TextDocumentPositionParams
     endpoint: string
   }): Promise<{ helpHTML?: string }> {
-    const leafNode = this.uriToTreeSitterTrees[
-      params.textDocument.uri
-    ].rootNode.descendantForPosition({
+    const analyzedDocument = this.uriToAnalyzedDocument[params.textDocument.uri]
+
+    const leafNode = analyzedDocument?.tree.rootNode.descendantForPosition({
       row: params.position.line,
       column: params.position.character,
     })
 
+    if (!leafNode || !analyzedDocument) {
+      return {}
+    }
+
     // explainshell needs the whole command, not just the "word" (tree-sitter
     // parlance) that the user hovered over. A relatively successful heuristic
     // is to simply go up one level in the AST. If you go up too far, you'll
@@ -223,10 +372,9 @@ export default class Analyzer {
       return {}
     }
 
-    const cmd = this.uriToFileContent[params.textDocument.uri].slice(
-      interestingNode.startIndex,
-      interestingNode.endIndex,
-    )
+    const cmd = analyzedDocument.document
+      .getText()
+      .slice(interestingNode.startIndex, interestingNode.endIndex)
     type ExplainshellResponse = {
       matches?: Array<{ helpHTML: string; start: number; end: number }>
     }
@@ -242,8 +390,7 @@ export default class Analyzer {
       return {}
     } else {
       const offsetOfMousePointerInCommand =
-        this.uriToTextDocument[params.textDocument.uri].offsetAt(params.position) -
-        interestingNode.startIndex
+        analyzedDocument.document.offsetAt(params.position) - interestingNode.startIndex
 
       const match = explainshellResponse.matches.find(
         (helpItem) =>
@@ -255,235 +402,6 @@ export default class Analyzer {
     }
   }
 
-  /**
-   * Find all the locations where something named name has been defined.
-   */
-  public findReferences(name: string): LSP.Location[] {
-    const uris = Object.keys(this.uriToTreeSitterTrees)
-    return flattenArray(uris.map((uri) => this.findOccurrences(uri, name)))
-  }
-
-  /**
-   * Find all occurrences of name in the given file.
-   * It's currently not scope-aware.
-   */
-  public findOccurrences(uri: string, query: string): LSP.Location[] {
-    const tree = this.uriToTreeSitterTrees[uri]
-    const contents = this.uriToFileContent[uri]
-
-    const locations: LSP.Location[] = []
-
-    TreeSitterUtil.forEach(tree.rootNode, (n) => {
-      let name: null | string = null
-      let range: null | LSP.Range = null
-
-      if (TreeSitterUtil.isReference(n)) {
-        const node = n.firstNamedChild || n
-        name = contents.slice(node.startIndex, node.endIndex)
-        range = TreeSitterUtil.range(node)
-      } else if (TreeSitterUtil.isDefinition(n)) {
-        const namedNode = n.firstNamedChild
-        if (namedNode) {
-          name = contents.slice(namedNode.startIndex, namedNode.endIndex)
-          range = TreeSitterUtil.range(namedNode)
-        }
-      }
-
-      if (name === query && range !== null) {
-        locations.push(LSP.Location.create(uri, range))
-      }
-    })
-
-    return locations
-  }
-
-  /**
-   * Find all symbol definitions in the given file.
-   */
-  public findSymbolsForFile({ uri }: { uri: string }): LSP.SymbolInformation[] {
-    const declarationsInFile = this.uriToDeclarations[uri] || {}
-    return flattenObjectValues(declarationsInFile)
-  }
-
-  /**
-   * Find symbol completions for the given word.
-   */
-  public findSymbolsMatchingWord({
-    exactMatch,
-    uri,
-    word,
-  }: {
-    exactMatch: boolean
-    uri: string
-    word: string
-  }): LSP.SymbolInformation[] {
-    const fileDeclarations = this.getReachableUriToDeclarations({ uri })
-
-    return Object.keys(fileDeclarations).reduce((symbols, uri) => {
-      const declarationsInFile = fileDeclarations[uri]
-      Object.keys(declarationsInFile).map((name) => {
-        const match = exactMatch ? name === word : name.startsWith(word)
-        if (match) {
-          declarationsInFile[name].forEach((symbol) => symbols.push(symbol))
-        }
-      })
-
-      return symbols
-    }, [] as LSP.SymbolInformation[])
-  }
-
-  /**
-   * Analyze the given document, cache the tree-sitter AST, and iterate over the
-   * tree to find declarations.
-   *
-   * Returns all, if any, syntax errors that occurred while parsing the file.
-   *
-   */
-  public analyze({
-    document,
-    uri, // NOTE: we don't use document.uri to make testing easier
-  }: {
-    document: TextDocument
-    uri: string
-  }): LSP.Diagnostic[] {
-    const contents = document.getText()
-
-    const tree = this.parser.parse(contents)
-
-    // TODO: would be nicer to save one map from uri to object containing all
-    // these fields.
-    this.uriToTextDocument[uri] = document
-    this.uriToTreeSitterTrees[uri] = tree
-    this.uriToDeclarations[uri] = {}
-    this.uriToFileContent[uri] = contents
-    this.uriToSourcedUris[uri] = sourcing.getSourcedUris({
-      fileContent: contents,
-      fileUri: uri,
-      rootPath: this.workspaceFolder,
-      tree,
-    })
-
-    const problems: LSP.Diagnostic[] = []
-
-    // TODO: move this somewhere
-    TreeSitterUtil.forEach(tree.rootNode, (n: Parser.SyntaxNode) => {
-      if (n.type === 'ERROR') {
-        problems.push(
-          LSP.Diagnostic.create(
-            TreeSitterUtil.range(n),
-            'Failed to parse expression',
-            LSP.DiagnosticSeverity.Error,
-          ),
-        )
-        return
-      } else if (TreeSitterUtil.isDefinition(n)) {
-        const named = n.firstNamedChild
-
-        if (named === null) {
-          return
-        }
-
-        const word = contents.slice(named.startIndex, named.endIndex)
-        const namedDeclarations = this.uriToDeclarations[uri][word] || []
-
-        const parent = TreeSitterUtil.findParent(
-          n,
-          (p) => p.type === 'function_definition',
-        )
-        const parentName =
-          parent && parent.firstNamedChild
-            ? contents.slice(
-                parent.firstNamedChild.startIndex,
-                parent.firstNamedChild.endIndex,
-              )
-            : '' // TODO: unsure what we should do here?
-
-        namedDeclarations.push(
-          LSP.SymbolInformation.create(
-            word,
-            this.treeSitterTypeToLSPKind[n.type],
-            TreeSitterUtil.range(n),
-            uri,
-            parentName,
-          ),
-        )
-        this.uriToDeclarations[uri][word] = namedDeclarations
-      }
-    })
-
-    function findMissingNodes(node: Parser.SyntaxNode) {
-      if (node.isMissing()) {
-        problems.push(
-          LSP.Diagnostic.create(
-            TreeSitterUtil.range(node),
-            `Syntax error: expected "${node.type}" somewhere in the file`,
-            LSP.DiagnosticSeverity.Warning,
-          ),
-        )
-      } else if (node.hasError()) {
-        node.children.forEach(findMissingNodes)
-      }
-    }
-
-    findMissingNodes(tree.rootNode)
-
-    return problems
-  }
-
-  public findAllSourcedUris({ uri }: { uri: string }): Set<string> {
-    const allSourcedUris = new Set<string>([])
-
-    const addSourcedFilesFromUri = (fromUri: string) => {
-      const sourcedUris = this.uriToSourcedUris[fromUri]
-
-      if (!sourcedUris) {
-        return
-      }
-
-      sourcedUris.forEach((sourcedUri) => {
-        if (!allSourcedUris.has(sourcedUri)) {
-          allSourcedUris.add(sourcedUri)
-          addSourcedFilesFromUri(sourcedUri)
-        }
-      })
-    }
-
-    addSourcedFilesFromUri(uri)
-
-    return allSourcedUris
-  }
-
-  /**
-   * Find the node at the given point.
-   */
-  private nodeAtPoint(
-    uri: string,
-    line: number,
-    column: number,
-  ): Parser.SyntaxNode | null {
-    const document = this.uriToTreeSitterTrees[uri]
-
-    if (!document?.rootNode) {
-      // Check for lacking rootNode (due to failed parse?)
-      return null
-    }
-
-    return document.rootNode.descendantForPosition({ row: line, column })
-  }
-
-  /**
-   * Find the full word at the given point.
-   */
-  public wordAtPoint(uri: string, line: number, column: number): string | null {
-    const node = this.nodeAtPoint(uri, line, column)
-
-    if (!node || node.childCount > 0 || node.text.trim() === '') {
-      return null
-    }
-
-    return node.text.trim()
-  }
-
   /**
    * Find the name of the command at the given point.
    */
@@ -511,7 +429,10 @@ export default class Analyzer {
    * Find a block of comments above a line position
    */
   public commentsAbove(uri: string, line: number): string | null {
-    const doc = this.uriToTextDocument[uri]
+    const doc = this.uriToAnalyzedDocument[uri]?.document
+    if (!doc) {
+      return null
+    }
 
     const commentBlock = []
 
@@ -558,28 +479,33 @@ export default class Analyzer {
     return null
   }
 
-  public getAllVariableSymbols({ uri }: { uri: string }): LSP.SymbolInformation[] {
-    return this.getAllSymbols({ uri }).filter(
-      (symbol) => symbol.kind === LSP.SymbolKind.Variable,
-    )
+  /**
+   * Find the full word at the given point.
+   */
+  public wordAtPoint(uri: string, line: number, column: number): string | null {
+    const node = this.nodeAtPoint(uri, line, column)
+
+    if (!node || node.childCount > 0 || node.text.trim() === '') {
+      return null
+    }
+
+    return node.text.trim()
   }
 
   public setIncludeAllWorkspaceSymbols(includeAllWorkspaceSymbols: boolean): void {
     this.includeAllWorkspaceSymbols = includeAllWorkspaceSymbols
   }
 
-  private getReachableUriToDeclarations({
-    uri: fromUri,
-  }: { uri?: string } = {}): FileDeclarations {
+  // Private methods
+  private getReachableUris({ uri: fromUri }: { uri?: string } = {}): string[] {
     if (!fromUri || this.includeAllWorkspaceSymbols) {
-      return this.uriToDeclarations
+      return Object.keys(this.uriToAnalyzedDocument)
     }
 
     const uris = [fromUri, ...Array.from(this.findAllSourcedUris({ uri: fromUri }))]
 
-    // for each uri (sourced and base file) we find the declarations
-    return uris.reduce((fileDeclarations, uri) => {
-      if (!this.uriToDeclarations[uri]) {
+    return uris.filter((uri) => {
+      if (!this.uriToAnalyzedDocument[uri]) {
         // Either the background analysis didn't run or the file is outside
         // the workspace. Let us try to analyze the file.
         try {
@@ -590,25 +516,115 @@ export default class Analyzer {
           })
         } catch (err) {
           this.console.log(`Error while analyzing sourced file ${uri}: ${err}`)
+          return false
         }
       }
 
-      fileDeclarations[uri] = this.uriToDeclarations[uri] || {}
-      return fileDeclarations
-    }, {} as FileDeclarations)
+      return true
+    })
   }
 
-  private getAllSymbols({ uri }: { uri?: string } = {}): LSP.SymbolInformation[] {
-    const fileDeclarations = this.getReachableUriToDeclarations({ uri })
+  /**
+   * Get all declaration symbols (function or variables) from the given file/position
+   * or from all files in the workspace. It will take into account the given position
+   * to filter out irrelevant symbols.
+   *
+   * Note that this can return duplicates across the workspace.
+   */
+  private getAllDeclarations({
+    uri: fromUri,
+    position,
+  }: { uri?: string; position?: LSP.Position } = {}): LSP.SymbolInformation[] {
+    return this.getReachableUris({ uri: fromUri }).reduce((symbols, uri) => {
+      const analyzedDocument = this.uriToAnalyzedDocument[uri]
 
-    return Object.keys(fileDeclarations).reduce((symbols, uri) => {
-      const declarationsInFile = fileDeclarations[uri]
-      Object.keys(declarationsInFile).forEach((name) => {
-        const declarationNames = this.uriToDeclarations[uri][name] || []
-        declarationNames.forEach((d) => symbols.push(d))
-      })
+      if (analyzedDocument) {
+        if (uri !== fromUri || !position) {
+          // We use the global declarations for external files or if we do not have a position
+          const { globalDeclarations } = analyzedDocument
+          Object.values(globalDeclarations).forEach((symbol) => symbols.push(symbol))
+        }
+
+        // For the current file we find declarations based on the current scope
+        if (uri === fromUri && position) {
+          const node = analyzedDocument.tree.rootNode?.descendantForPosition({
+            row: position.line,
+            column: position.character,
+          })
+
+          const localDeclarations = getLocalDeclarations({
+            node,
+            uri,
+          })
+
+          Object.keys(localDeclarations).map((name) => {
+            const symbolsMatchingWord = localDeclarations[name]
+
+            // Find the latest definition
+            let closestSymbol: LSP.SymbolInformation | null = null
+            symbolsMatchingWord.forEach((symbol) => {
+              // Skip if the symbol is defined in the current file after the requested position
+              if (symbol.location.range.start.line > position.line) {
+                return
+              }
+
+              if (
+                closestSymbol === null ||
+                symbol.location.range.start.line > closestSymbol.location.range.start.line
+              ) {
+                closestSymbol = symbol
+              }
+            })
+
+            if (closestSymbol) {
+              symbols.push(closestSymbol)
+            }
+          })
+        }
+      }
 
       return symbols
     }, [] as LSP.SymbolInformation[])
   }
+
+  public findAllSourcedUris({ uri }: { uri: string }): Set<string> {
+    const allSourcedUris = new Set<string>([])
+
+    const addSourcedFilesFromUri = (fromUri: string) => {
+      const sourcedUris = this.uriToAnalyzedDocument[fromUri]?.sourcedUris
+
+      if (!sourcedUris) {
+        return
+      }
+
+      sourcedUris.forEach((sourcedUri) => {
+        if (!allSourcedUris.has(sourcedUri)) {
+          allSourcedUris.add(sourcedUri)
+          addSourcedFilesFromUri(sourcedUri)
+        }
+      })
+    }
+
+    addSourcedFilesFromUri(uri)
+
+    return allSourcedUris
+  }
+
+  /**
+   * Find the node at the given point.
+   */
+  private nodeAtPoint(
+    uri: string,
+    line: number,
+    column: number,
+  ): Parser.SyntaxNode | null {
+    const tree = this.uriToAnalyzedDocument[uri]?.tree
+
+    if (!tree?.rootNode) {
+      // Check for lacking rootNode (due to failed parse?)
+      return null
+    }
+
+    return tree.rootNode.descendantForPosition({ row: line, column })
+  }
 }
