@@ -1,6 +1,6 @@
 'use strict'
 
-import * as LSP from 'vscode-languageserver'
+import * as LSP from 'vscode-languageserver/node'
 
 import BashServer from './server'
 
@@ -9,23 +9,22 @@ const pkg = require('../package')
 export function listen() {
   // Create a connection for the server.
   // The connection uses stdin/stdout for communication.
-  const connection: LSP.IConnection = LSP.createConnection(
+  const connection = LSP.createConnection(
     new LSP.StreamMessageReader(process.stdin),
     new LSP.StreamMessageWriter(process.stdout),
   )
 
   connection.onInitialize(
-    (params: LSP.InitializeParams): Promise<LSP.InitializeResult> => {
+    async (params: LSP.InitializeParams): Promise<LSP.InitializeResult> => {
       connection.console.log(`Initialized server v. ${pkg.version} for ${params.rootUri}`)
 
-      return BashServer.initialize(connection, params)
-        .then(server => {
-          server.register(connection)
-          return server
-        })
-        .then(server => ({
-          capabilities: server.capabilities(),
-        }))
+      const server = await BashServer.initialize(connection, params)
+
+      server.register(connection)
+
+      return {
+        capabilities: server.capabilities(),
+      }
     },
   )
 
