@@ -204,7 +204,7 @@ export default class BashServer {
   }
 
   private updateConfiguration(configObject: any): boolean {
-    if (typeof configObject === 'object') {
+    if (typeof configObject === 'object' && configObject !== null) {
       try {
         const newConfig = config.ConfigSchema.parse(configObject)
 
@@ -426,10 +426,16 @@ export default class BashServer {
       }
     }
 
+    const symbolsMatchingWord = this.analyzer.findDeclarationsMatchingWord({
+      exactMatch: true,
+      uri: currentUri,
+      word,
+      position: params.position,
+    })
     if (
       ReservedWords.isReservedWord(word) ||
       Builtins.isBuiltin(word) ||
-      this.executables.isExecutableOnPATH(word)
+      (this.executables.isExecutableOnPATH(word) && symbolsMatchingWord.length == 0)
     ) {
       const shellDocumentation = await getShellDocumentation({ word })
       if (shellDocumentation) {
@@ -437,11 +443,7 @@ export default class BashServer {
       }
     } else {
       const symbolDocumentation = deduplicateSymbols({
-        symbols: this.analyzer.findSymbolsMatchingWord({
-          exactMatch: true,
-          uri: currentUri,
-          word,
-        }),
+        symbols: symbolsMatchingWord,
         currentUri,
       })
         // do not return hover referencing for the current line
@@ -464,7 +466,7 @@ export default class BashServer {
     if (!word) {
       return null
     }
-    return this.analyzer.findDefinition({
+    return this.analyzer.findDeclarationLocations({
       position: params.position,
       uri: params.textDocument.uri,
       word,
@@ -472,13 +474,16 @@ export default class BashServer {
   }
 
   private onDocumentSymbol(params: LSP.DocumentSymbolParams): LSP.SymbolInformation[] {
+    // TODO: ideally this should return LSP.DocumentSymbol[] instead of LSP.SymbolInformation[]
+    // which is a hierarchy of symbols.
+    // https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#textDocument_documentSymbol
     this.connection.console.log(`onDocumentSymbol`)
-    return this.analyzer.findSymbolsForFile({ uri: params.textDocument.uri })
+    return this.analyzer.getDeclarationsForUri({ uri: params.textDocument.uri })
   }
 
   private onWorkspaceSymbol(params: LSP.WorkspaceSymbolParams): LSP.SymbolInformation[] {
     this.connection.console.log('onWorkspaceSymbol')
-    return this.analyzer.search(params.query)
+    return this.analyzer.findDeclarationsWithFuzzySearch(params.query)
   }
 
   private onDocumentHighlight(
@@ -557,11 +562,15 @@ export default class BashServer {
         ? []
         : this.getCompletionItemsForSymbols({
             symbols: shouldCompleteOnVariables
-              ? this.analyzer.getAllVariableSymbols({ uri: currentUri })
-              : this.analyzer.findSymbolsMatchingWord({
+              ? this.analyzer.getAllVariables({
+                  uri: currentUri,
+                  position: params.position,
+                })
+              : this.analyzer.findDeclarationsMatchingWord({
                   exactMatch: false,
                   uri: currentUri,
                   word,
+                  position: params.position,
                 }),
             currentUri,
           })
@@ -731,6 +740,7 @@ function deduplicateSymbols({
         ),
     )
 
+  // NOTE: it might be that uniqueBasedOnHash is not needed anymore
   return uniqueBasedOnHash([...symbolsCurrentFile, ...symbolsOtherFiles], getSymbolId)
 }
 
